@@ -12,10 +12,10 @@ my $external_stylesheets = ['https://codepen.io/chriddyp/pen/bWLwgP.css'];
 
 my $app = Dash->new(app_name => 'Dash Tutorial - 2 Layout', external_stylesheets=>$external_stylesheets);
 
-$app->layout( html->Div(children=>[
-    html->H1(children=>'Hello Dash'),
+$app->layout( html->Div([
+    html->H1('Hello Dash'),
 
-    html->Div(children=>'
+    html->Div('
         Dash: A web application framework for Python, R & Perl!.
     '),
 
